@@ -604,11 +604,8 @@ tracker_miner_manager_new_full (gboolean   auto_start,
 	                          &inner_error,
 	                          "auto-start", auto_start,
 	                          NULL);
-	if (!manager) {
-		g_critical ("Couldn't create new TrackerMinerManager: '%s'",
-		            inner_error ? inner_error->message : "unknown error");
-		g_clear_error (&inner_error);
-	}
+	if (inner_error)
+		g_propagate_error (error, inner_error);
 
 	return manager;
 }
@@ -907,6 +904,10 @@ tracker_miner_manager_pause (TrackerMinerManager *manager,
  * where the calling process has a risk of crashing without resuming
  * the pause.
  *
+ * NOTE: If you call g_object_unref() on the @manager before you
+ * intend to resume the pause and it finalizes, it will automatically
+ * resume.
+ *
  * Returns: %TRUE if the miner was paused successfully, otherwise
  * %FALSE.
  *
