@@ -26,10 +26,6 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <unistd.h>
-#if defined(__linux__)
-#include <linux/sched.h>
-#endif
-#include <sched.h>
 
 #include <glib.h>
 #include <glib-object.h>
@@ -39,6 +35,8 @@
 #include <libtracker-common/tracker-log.h>
 #include <libtracker-common/tracker-ontologies.h>
 #include <libtracker-common/tracker-file-utils.h>
+#include <libtracker-common/tracker-sched.h>
+#include <libtracker-common/tracker-enums.h>
 
 #include <libtracker-miner/tracker-miner.h>
 
@@ -46,7 +44,6 @@
 
 #include "tracker-config.h"
 #include "tracker-marshal.h"
-#include "tracker-miner-userguides.h"
 #include "tracker-miner-applications.h"
 #include "tracker-miner-files.h"
 #include "tracker-miner-files-index.h"
@@ -64,26 +61,6 @@
 
 #define SECONDS_PER_DAY 60 * 60 * 24
 
-#define OPTION_DISABLE_FILES "files"
-#define OPTION_DISABLE_APPLICATIONS "applications"
-#define OPTION_DISABLE_USERGUIDES "userguides"
-
-typedef enum {
-	DISABLE_NONE,
-	DISABLE_FILES,
-	DISABLE_APPLICATIONS,
-#ifdef HAVE_MAEMO
-	DISABLE_USERGUIDES,
-#endif /* HAVE_MAEMO */
-} DisableOption;
-
-static gboolean miner_disable_option_arg_func (const gchar  *option_value,
-                                               const gchar  *value,
-                                               gpointer      data,
-                                               GError      **error);
-static void     miner_handle_next             (void);
-
-
 static GMainLoop *main_loop;
 static GSList *miners;
 static GSList *current_miner;
@@ -93,8 +70,6 @@ static gint verbosity = -1;
 static gint initial_sleep = -1;
 static gboolean no_daemon;
 static gchar *eligible;
-static GArray *disable_options = NULL;
-/* static DisableOption disable_option[] = { 0 }; */
 static gboolean version;
 static guint miners_timeout_id = 0;
 
@@ -117,16 +92,6 @@ static GOptionEntry entries[] = {
 	  G_OPTION_ARG_FILENAME, &eligible,
 	  N_("Checks if FILE is eligible for being mined based on configuration"),
 	  N_("FILE") },
-	{ "disable-miner", 'd', G_OPTION_FLAG_OPTIONAL_ARG,
-	  G_OPTION_ARG_CALLBACK, miner_disable_option_arg_func,
-	  N_("Disable miners started as part of this process, options include"
-	     ": '" OPTION_DISABLE_FILES "'"
-	     ", '" OPTION_DISABLE_APPLICATIONS "'"
-#ifdef HAVE_MAEMO
-	     ", '" OPTION_DISABLE_USERGUIDES "'"
-#endif /* HAVE_MAEMO */
-	  ),
-	  N_("MINER") },
 	{ "version", 'V', 0,
 	  G_OPTION_ARG_NONE, &version,
 	  N_("Displays version information"),
@@ -134,80 +99,14 @@ static GOptionEntry entries[] = {
 	{ NULL }
 };
 
-static gboolean
-miner_disable_option_arg_func (const gchar  *option_value,
-                               const gchar  *value,
-                               gpointer      data,
-                               GError      **error)
-{
-	DisableOption option;
-	gchar *value_casefold;
-	gboolean found = FALSE;
-	gint i;
-
-	if (!value || *value == '\0') {
-		/* Show help */
-#ifdef HAVE_MAEMO
-		g_set_error_literal (error,
-		                     G_OPTION_ERROR,
-		                     G_OPTION_ERROR_FAILED,
-		                     "A value is required, either "
-		                     "'" OPTION_DISABLE_FILES "', "
-		                     "'" OPTION_DISABLE_APPLICATIONS "' or "
-		                     "'" OPTION_DISABLE_USERGUIDES "'");
-#else  /* HAVE_MAEMO */
-		g_set_error_literal (error,
-		                     G_OPTION_ERROR,
-		                     G_OPTION_ERROR_FAILED,
-		                     "A value is required, either "
-		                     "'" OPTION_DISABLE_FILES "', "
-		                     "'" OPTION_DISABLE_APPLICATIONS "'");
-#endif /* HAVE_MAEMO */
-		return FALSE;
-	}
-
-	value_casefold = g_utf8_casefold (value, -1);
-
-	if (strcmp (value_casefold, OPTION_DISABLE_FILES) == 0) {
-		option = DISABLE_FILES;
-	} else if (strcmp (value_casefold, OPTION_DISABLE_APPLICATIONS) == 0) {
-		option = DISABLE_APPLICATIONS;
-#ifdef HAVE_MAEMO
-	} else if (strcmp (value_casefold, OPTION_DISABLE_USERGUIDES) == 0) {
-		option = DISABLE_USERGUIDES;
-#endif /* HAVE_MAEMO */
-	} else {
-		g_set_error (error,
-		             G_OPTION_ERROR,
-		             G_OPTION_ERROR_FAILED,
-		             "Miner '%s' is not recognized",
-		             value);
-		return FALSE;
-	}
-
-	g_free (value_casefold);
-
-	/* Check we didn't already disable this miner */
-	for (i = 0; i < disable_options->len; i++) {
-		if (g_array_index (disable_options, gint, i) == option) {
-			found = TRUE;
-			break;
-		}
-	}
-
-	if (!found) {
-		g_array_append_val (disable_options, option);
-	}
-
-	return TRUE;
-}
-
 static void
 sanity_check_option_values (TrackerConfig *config)
 {
 	g_message ("General options:");
 	g_message ("  Verbosity  ............................  %d",
 	           tracker_config_get_verbosity (config));
+	g_message ("  Sched Idle  ...........................  %d",
+	           tracker_config_get_sched_idle (config));
 	g_message ("  Initial Sleep  ........................  %d",
 	           tracker_config_get_initial_sleep (config));
 
@@ -273,8 +172,15 @@ initialize_signal_handler (void)
 }
 
 static void
-initialize_priority (void)
+initialize_priority_and_scheduling (TrackerSchedIdle sched_idle,
+                                    gboolean         first_time_index)
 {
+	/* Set CPU priority */
+	if (sched_idle == TRACKER_SCHED_IDLE_ALWAYS ||
+	    (sched_idle == TRACKER_SCHED_IDLE_FIRST_INDEX && first_time_index)) {
+		tracker_sched_idle ();
+	}
+
 	/* Set disk IO priority and scheduling */
 	tracker_ioprio_init ();
 
@@ -286,6 +192,8 @@ initialize_priority (void)
 	 * Stupid...
 	 */
 
+	g_message ("Setting priority nice level to 19");
+
 	errno = 0;
 	if (nice (19) == -1 && errno != 0) {
 		const gchar *str = g_strerror (errno);
@@ -340,62 +248,6 @@ should_crawl (TrackerConfig *config,
 	}
 }
 
-static gboolean
-miner_disabled_check (void)
-{
-	gchar *name = NULL;
-	gchar *name_casefold;
-	gboolean disabled = FALSE;
-	gint i;
-
-	if (!disable_options || disable_options->len < 1) {
-		return FALSE;
-	}
-
-	g_object_get (current_miner->data, "name", &name, NULL);
-
-	if (!name || *name == '\0') {
-		g_warning ("Expected miner to have 'name' property set. "
-		           "Can not disable this particular miner as a result...");
-		return FALSE;
-	}
-
-	name_casefold = g_utf8_casefold (name, -1);
-
-	for (i = 0; i < disable_options->len && !disabled; i++) {
-		DisableOption o = (gint) g_array_index (disable_options, gint, i);
-
-		switch (o) {
-		case DISABLE_FILES:
-			disabled = strcmp (name_casefold, OPTION_DISABLE_FILES) == 0;
-			break;
-		case DISABLE_APPLICATIONS:
-			disabled = strcmp (name_casefold, OPTION_DISABLE_APPLICATIONS) == 0;
-			break;
-
-#ifdef HAVE_MAEMO
-		case DISABLE_USERGUIDES:
-			disabled = strcmp (name_casefold, OPTION_DISABLE_USERGUIDES) == 0;
-			break;
-#endif /* HAVE_MAEMO */
-
-		default:
-			g_assert_not_reached ();
-			break;
-		}
-	}
-
-	if (disabled) {
-		g_message ("Miner '%s' was disabled on the command line, moving to next...", name);
-		miner_handle_next ();
-	}
-
-	g_free (name_casefold);
-	g_free (name);
-
-	return disabled;
-}
-
 static void
 miner_handle_next (void)
 {
@@ -424,11 +276,6 @@ miner_handle_next (void)
 		return;
 	}
 
-	/* Check disabled miners */
-	if (miner_disabled_check ()) {
-		return;
-	}
-
 	if (!tracker_miner_is_started (current_miner->data)) {
 		g_message ("Starting next miner...");
 		tracker_miner_start (current_miner->data);
@@ -809,11 +656,6 @@ miner_needs_check (TrackerMiner *miner,
 			return TRUE;
 		}
 
-		/* FIXME: We currently don't check the applications
-		 *  miner OR the userguides miner if we are finished
-		 * before returning TRUE/FALSE here, should we?
-		 */
-
 		/* We consider the miner finished */
 		return FALSE;
 	} else {
@@ -835,9 +677,6 @@ main (gint argc, gchar *argv[])
 	TrackerConfig *config;
 	TrackerMiner *miner_applications, *miner_files;
 	TrackerMinerFilesIndex *miner_files_index;
-#ifdef HAVE_MAEMO
-	TrackerMiner *miner_userguides;
-#endif /* HAVE_MAEMO */
 	GOptionContext *context;
 	GError *error = NULL;
 	gchar *log_filename = NULL;
@@ -866,27 +705,17 @@ main (gint argc, gchar *argv[])
 	 */
 	context = g_option_context_new (_("- start the tracker indexer"));
 
-	disable_options = g_array_new (FALSE, FALSE, sizeof (gint));
 	g_option_context_add_main_entries (context, entries, NULL);
 	g_option_context_parse (context, &argc, &argv, &error);
 	g_option_context_free (context);
 
-	if (error) {
-		g_printerr ("%s\n", error->message);
-		g_error_free (error);
-		g_array_free (disable_options, TRUE);
-		return EXIT_FAILURE;
-	}
-
 	if (version) {
 		g_print ("\n" ABOUT "\n" LICENSE "\n");
-		g_array_free (disable_options, TRUE);
 		return EXIT_SUCCESS;
 	}
 
 	if (eligible) {
 		check_eligible ();
-		g_array_free (disable_options, TRUE);
 		return EXIT_SUCCESS;
 	}
 
@@ -894,9 +723,6 @@ main (gint argc, gchar *argv[])
 
 	initialize_signal_handler ();
 
-	/* This makes sure we don't steal all the system's resources */
-	initialize_priority ();
-
 	/* Initialize logging */
 	config = tracker_config_new ();
 
@@ -915,6 +741,10 @@ main (gint argc, gchar *argv[])
 
 	sanity_check_option_values (config);
 
+	/* This makes sure we don't steal all the system's resources */
+	initialize_priority_and_scheduling (tracker_config_get_sched_idle (config),
+	                                    tracker_db_manager_get_first_index_done () == FALSE);
+
 	main_loop = g_main_loop_new (NULL, FALSE);
 
 	g_message ("Checking if we're running as a daemon:");
@@ -929,7 +759,6 @@ main (gint argc, gchar *argv[])
 		            error ? error->message : "unknown error");
 		g_object_unref (config);
 		tracker_log_shutdown ();
-		g_array_free (disable_options, TRUE);
 		return EXIT_FAILURE;
 	}
 
@@ -943,7 +772,6 @@ main (gint argc, gchar *argv[])
 		g_object_unref (config);
 		g_object_unref (miner_files);
 		tracker_log_shutdown ();
-		g_array_free (disable_options, TRUE);
 		return EXIT_FAILURE;
 	}
 
@@ -956,7 +784,6 @@ main (gint argc, gchar *argv[])
 		tracker_writeback_shutdown ();
 		g_object_unref (config);
 		tracker_log_shutdown ();
-		g_array_free (disable_options, TRUE);
 		return EXIT_FAILURE;
 	}
 
@@ -968,27 +795,9 @@ main (gint argc, gchar *argv[])
 		tracker_writeback_shutdown ();
 		g_object_unref (config);
 		tracker_log_shutdown ();
-		g_array_free (disable_options, TRUE);
 		return EXIT_FAILURE;
 	}
 
-#ifdef HAVE_MAEMO
-	/* Create miner for userguides */
-	miner_userguides = tracker_miner_userguides_new (&error);
-	if (!miner_userguides) {
-		g_critical ("Couldn't create new User Guides miner: '%s'",
-		            error ? error->message : "unknown error");
-		g_object_unref (miner_applications);
-		g_object_unref (miner_files);
-		g_object_unref (miner_files_index);
-		tracker_writeback_shutdown ();
-		g_object_unref (config);
-		tracker_log_shutdown ();
-		g_array_free (disable_options, TRUE);
-		return EXIT_FAILURE;
-	}
-#endif /* HAVE_MAEMO */
-
 	/* Check if we should crawl and if we should force mtime
 	 * checking based on the config.
 	 */
@@ -1029,11 +838,6 @@ main (gint argc, gchar *argv[])
 	/* Configure applications miner */
 	tracker_miner_fs_set_initial_crawling (TRACKER_MINER_FS (miner_applications), do_crawling);
 
-#ifdef HAVE_MAEMO
-	/* Configure userguides miner */
-	tracker_miner_fs_set_initial_crawling (TRACKER_MINER_FS (miner_userguides), do_crawling);
-#endif /* HAVE_MAEMO */
-
 	/* If a locale change was detected, always do mtime checks */
 	if (tracker_miner_applications_detect_locale_changed (miner_applications)) {
 		if (!do_mtime_checking)
@@ -1043,33 +847,11 @@ main (gint argc, gchar *argv[])
 		tracker_miner_fs_set_mtime_checking (TRACKER_MINER_FS (miner_applications), do_mtime_checking);
 	}
 
-
-#ifdef HAVE_MAEMO
-	/* If a locale change was detected, always do mtime checks */
-	if (tracker_miner_userguides_detect_locale_changed (miner_userguides)) {
-		if (!do_mtime_checking)
-			g_debug ("Forcing mtime check in userguides miner as locale change was detected");
-		tracker_miner_fs_set_mtime_checking (TRACKER_MINER_FS (miner_userguides), TRUE);
-	} else {
-		tracker_miner_fs_set_mtime_checking (TRACKER_MINER_FS (miner_userguides), do_mtime_checking);
-	}
-#endif /* HAVE_MAEMO */
-
 	g_signal_connect (miner_applications, "finished",
 	                  G_CALLBACK (miner_finished_cb),
 	                  NULL);
 
-#ifdef HAVE_MAEMO
-	g_signal_connect (miner_userguides, "finished",
-			  G_CALLBACK (miner_finished_cb),
-			  NULL);
-#endif /* HAVE_MAEMO */
-
 	/* Setup miners, applications first in list */
-#ifdef HAVE_MAEMO
-	miners = g_slist_prepend (miners, miner_userguides);
-#endif /* HAVE_MAEMO */
-
 	miners = g_slist_prepend (miners, miner_files);
 	miners = g_slist_prepend (miners, miner_applications);
 
@@ -1087,13 +869,7 @@ main (gint argc, gchar *argv[])
 	if (miners_timeout_id == 0 &&
 	    !miner_needs_check (miner_files, store_available) &&
 	    !miner_needs_check (miner_applications, store_available)) {
-#ifdef HAVE_MAEMO
-		if (!miner_needs_check (miner_userguides, store_available)) {
-			tracker_db_manager_set_need_mtime_check (FALSE);
-		}
-#else  /* HAVE_MAEMO */
 		tracker_db_manager_set_need_mtime_check (FALSE);
-#endif /* HAVE_MAEMO */
 	}
 
 	g_main_loop_unref (main_loop);
@@ -1108,8 +884,6 @@ main (gint argc, gchar *argv[])
 	tracker_writeback_shutdown ();
 	tracker_log_shutdown ();
 
-	g_array_free (disable_options, TRUE);
-
 	g_print ("\nOK\n\n");
 
 	return EXIT_SUCCESS;
