@@ -311,6 +311,9 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 	filename = g_file_get_path (file);
 
 	fd = g_open (filename, O_RDONLY | O_NOATIME, 0);
+	if (fd == -1 && errno == EPERM) {
+		fd = g_open (filename, O_RDONLY, 0);
+	}
 
 	if (fd == -1) {
 		g_warning ("Could not open pdf file '%s': %s\n",
@@ -360,6 +363,7 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 
 			g_error_free (error);
 			g_free (uri);
+			close (fd);
 
 			return TRUE;
 		} else {
@@ -369,6 +373,7 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 
 			g_error_free (error);
 			g_free (uri);
+			close (fd);
 
 			return FALSE;
 		}
@@ -379,6 +384,7 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 		           "NULL returned without an error",
 		           uri);
 		g_free (uri);
+		close (fd);
 		return FALSE;
 	}
 
