@@ -69,7 +69,7 @@ PREFIX = "/usr"
 # So we do the translation by hand in the expandvars function
 #
 RAW_EXEC_PREFIX = "${prefix}"
-RAW_EXEC_DIR = "${exec_prefix}/libexec"
+RAW_EXEC_DIR = "${exec_prefix}/lib"
 RAW_DATA_DIR = "${datarootdir}"
 RAW_DATAROOT_DIR = "${prefix}/share"
 RAW_BINDIR = "${exec_prefix}/bin"
@@ -79,8 +79,17 @@ DATADIR = os.path.normpath (expandvars (RAW_DATA_DIR))
 BINDIR = os.path.normpath (expandvars (RAW_BINDIR))
                             
 haveMaemo = ("#" == "")
-haveUpstart = ("" == "")
+haveUpstart = ("#" == "")
 disableJournal = ("#" == "")
 
-TEST_TMP_DIR = os.path.join (os.environ["HOME"], ".tracker-tests")
-        
+TEST_TMP_DIR = os.path.join (os.environ["HOME"], "tracker-tests")
+
+TEST_MONITORED_TMP_DIR = TEST_TMP_DIR
+
+if TEST_TMP_DIR.startswith('/tmp'):
+	if os.environ.has_key('REAL_HOME'):
+		TEST_MONITORED_TMP_DIR = os.path.join (os.environ["REAL_HOME"], "tracker-tests")
+	else:
+		print ("HOME is in the /tmp prefix - this will cause tests that rely " +
+		       "on filesystem monitoring to fail as changes in that prefix are " +
+		       "ignored.")
